/*
    ES Knot
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#include <coemain.h>
#include "knotappview.h"
#include <flogger.h>
#include "knot.h"
#include <hal.h>
#include <eikenv.h>
#include <GLES/gl.h>
#include <aknutils.h>
#include <aknnotewrappers.h>
#include <stdlib.h>

void debug(const char *str) {
	TBuf<100> buf;
	while (*str != '\0')
		buf.Append(*str++);
	RFileLogger::WriteFormat(_L("debug"), _L("knotdbg.txt"), EFileLoggingModeAppend, _L("%S"), &buf);
}

CKnotAppView* CKnotAppView::NewL(const TRect& aRect) {
	CKnotAppView* self = CKnotAppView::NewLC(aRect);
	CleanupStack::Pop(self);
	return self;
}

CKnotAppView* CKnotAppView::NewLC(const TRect& aRect) {
	CKnotAppView* self = new(ELeave) CKnotAppView();
	CleanupStack::PushL(self);
	self->ConstructL(aRect, EFullscreen, EEGLWindow, EFalse);
	return self;
}

CKnotAppView::CKnotAppView() {
}

void CKnotAppView::DeinitGC() {
	if (iFont && iGc)
		iGc->DiscardFont();
}

void CKnotAppView::InitGC() {
//	iFont = LatinBold13();
	iFont = LatinPlain12();
//	iFont = NumberPlain5();
	iGc->UseFont(iFont);
}

CKnotAppView::~CKnotAppView() {
	DeinitGC();
	delete iKnot;

	CloseSTDLIB();
}

void CKnotAppView::ConstructAppL() {
	iKnot = new(ELeave) Knot(Rect().Width(), Rect().Height());
	iKnot->setUpdateVertices(true);
	iKnot->setTexMode(Knot::TEXSPEC);
}

void CKnotAppView::ReinitApp() {
	if (iKnot)
		iKnot->reinit();
}

void CKnotAppView::SizeChanged() {
	if (iKnot)
		iKnot->setSize(Rect().Width(), Rect().Height());

	CGLAppView::SizeChanged();
}

void CKnotAppView::UpdateAndDraw() {
	iKnot->draw(iTime);

	FinishGLDrawing();

	if (iDrawMode != EEGLWindow) {
		eglWaitGL();
		TPoint point(2, 2 + iFont->HeightInPixels());
		TBuf<20> buf;
		buf.Format(_L("%d fps"), iLastFPS);
		iGc->SetPenColor(KRgbWhite);
		iGc->SetDrawMode(CGraphicsContext::EDrawModeXOR);
		iGc->DrawText(buf, point);
		TInt ticklength = iRelTick - iFrameTick[iFrame % iFrameTickLength];
		buf.Format(_L("%d ticks/%d frames"), ticklength, iFrameTickLength);
		point.iX = Rect().Width() - 2 - iFont->TextWidthInPixels(buf);
		iGc->DrawText(buf, point);
		eglWaitNative(EGL_CORE_NATIVE_ENGINE);
	}
}

void CKnotAppView::FPSUpdated() {
	 RFileLogger::WriteFormat(_L("debug"), _L("knot.txt"), EFileLoggingModeAppend, _L("%d fps"), iLastFPS);
}

TKeyResponse CKnotAppView::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType) {
	if (aType != EEventKeyDown)
		return EKeyWasNotConsumed;

	if (aKeyEvent.iScanCode == EStdKeyDevice3) {
		if (iKnot)
			iKnot->startMorph();
		return EKeyWasConsumed;
	}
	return EKeyWasNotConsumed;
}

