#include <SDL.h>
#include <GLES/gl.h>
#include <stdio.h>
#include <stdlib.h>
#include "knot.h"

#define WIDTH 176
#define HEIGHT 208

extern "C" GL_API int GL_APIENTRY _dgles_load_library(void *, void *(*)(void *, const char *));

void debug(const char *str) {
	printf("%s\n", str);
}

static void *proc_loader(void *h, const char *name) {
	return SDL_GL_GetProcAddress(name);
}

int main(int argc, char *argv[]) {
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) < 0) {
		fprintf(stderr, "SDL error: %s\n", SDL_GetError());
		return 1;
	}
	atexit(SDL_Quit);

	int err;
#if defined(WIN32)
	err = SDL_GL_LoadLibrary("opengl32.dll");
#elif defined(__linux__) || defined(__FreeBSD__)
	err = SDL_GL_LoadLibrary("libGL.so");
#elif defined(__APPLE__)
	err = SDL_GL_LoadLibrary("/usr/X11R6/lib/libGL.dylib");
#else
#error Your platform is not supported
	err = 1;
#endif
	if (err != 0) {
		fprintf(stderr, "Can't load OpenGL library: %s\n", SDL_GetError());
		return 1;
	}

	SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE, 0);
	SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE, 0);
	SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE, 0);
	SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE, 0);

	if (SDL_SetVideoMode(WIDTH, HEIGHT, 0, SDL_OPENGL | SDL_RESIZABLE) == NULL) {
		fprintf(stderr, "%s\n", SDL_GetError());
		return 1;
	}

	_dgles_load_library(NULL, proc_loader);

	SDL_WM_SetCaption("ESKnot", NULL);

	Knot *k = new Knot(WIDTH, HEIGHT);
	bool update = false;
	bool glow = false;
	bool radial = false;
	bool blur = false;

	Uint32 starttime = SDL_GetTicks();

	while (1) {
		SDL_Event event;
		bool done = false;
		while (SDL_PollEvent(&event)) {
			if (event.type == SDL_QUIT) {
				done = true;
			} else if (event.type == SDL_KEYDOWN) {
				switch (event.key.keysym.sym) {
				case SDLK_ESCAPE:
					done = true;
					break;
				case SDLK_SPACE:
					k->startMorph();
					break;
				case SDLK_1:
					k->setTexMode(Knot::UNTEXTURED);
					break;
				case SDLK_2:
					k->setTexMode(Knot::PLAINTEX);
					break;
				case SDLK_3:
					k->setTexMode(Knot::TEXSPEC);
					break;
				case SDLK_4:
					k->setTexMode(Knot::CELSHADED);
					break;
				case SDLK_u:
					update = !update;
					k->setUpdateVertices(update);
					break;
				case SDLK_r:
					radial = !radial;
					glow = false;
					k->setRadialBlur(radial);
					k->setRadialGlow(glow);
					break;
				case SDLK_g:
					glow = !glow;
					radial = glow;
					k->setRadialBlur(radial);
					k->setRadialGlow(glow);
					break;
				case SDLK_b:
					blur = !blur;
					k->setMotionBlur(blur);
					break;
				default:
					break;
				}
			} else if (event.type == SDL_VIDEORESIZE) {
				if (SDL_SetVideoMode(event.resize.w, event.resize.h, 0, SDL_OPENGL | SDL_RESIZABLE) == NULL) {
					fprintf(stderr, "%s\n", SDL_GetError());
					return 1;
				}
				k->setSize(event.resize.w, event.resize.h);
				k->reinit();
			}
		}
		if (done) break;

		k->draw(SDL_GetTicks() - starttime);

		glFlush();
		glFinish();
		SDL_GL_SwapBuffers();
		SDL_Delay(1);
	}

	delete k;
	return 0;
}

